Portal Client for Mobile
====================

Portal client Javascript library for use within the Backbase Mobile SDK. A similar implementation exists for
[desktop/web](https://stash.backbase.com/projects/CXP-WEB/repos/portal-client-web/browse)

##Artifacts

The build creates two artifacts, the regular and minified build (plus sourcemap):

```
dist/web-sdk-mobile.js
dist/web-sdk-mobile.min.js
```

This build will run on both **iOS** and **Android** webviews. Rendering specific decisions are made at runtime.

It's also available via NPM:

```
npm install portal-client-mobile@release --registry http://npm-cxp.backbase.dev:4873
```

<small>The `node_modules` directory might cause XCode to crash. If so, run `gulp postbuild` to clear this directory
once the build is complete.</small>

### Peer dependencies

Portal5 Client Shim is required for CXP 5 widget compatibility. For convenience, `portal-client-shim.js` is also resolved via NPM and copied to the `dist` directory as part of the Gulp build:

```
dist/portal-client-shim.js
```

### Polyfills

The CXP Web APIs use the browser-native _Promise_, _Fetch_ and Internationalization APIs.

* [For **Android <= 4.4** a polyfill is required for **Promises**](http://caniuse.com/#search=promise)
* [For **iOS Safari and Android <= 4.4.4** a polyfill is required for **Fetch API**.](http://caniuse.com/#search=fetch)
* [For **iOS Safari and Android <= 4.3** a polyfill is required for the **Internationalization API**](http://caniuse.com/#search=intl)

For convenience these polyfills are also resolved via NPM and copied to the `dist` directory as part of the Gulp build:

```
dist/promise.min.js
dist/fetch.js
dist/Intl.min.js
dist/en.json
```

Additionally, to fully polyfill the **Internationalization API**, all the relevant locale data should also be included.
The localization files for can be found at: `node_modules/intl-feature/bower_components/intl/locale-data`

##Demo

To run a standalone demo do:

```
npm install
npm start
```

Then, to emulate a mobile-like environment where the resources are served from the filesystem, open `/e2e/index.html`
from the filesystem in your browser. You will need to pass some flags to overcome the security restrictions associated
with this. You command should look something like this:

```
google-chrome --allow-file-access-from-files ./e2e/index.html
```

You should see that widget resources are served from the filesystem, while remote request are sent to the running Node
server on port 3000. The running server acts as an MBaaS model endpoint stub and also served dummy content.

Unfortunately on non-mobile filesystems we cannot set cookies, so sending cookies to remote hosts cannot be emulated.

##Usage

Start rendering by calling the globally exposed function `cxp.mobile.render()`

```javascript
    var model = {
        "id": "12345",
        "name": "simple-widget",
        "content": {
            "src": "$(contextRoot)/static/widgets/simple-widget/index.html"
        },
        "features" : []
    };

    var syncPrefs = {
        selectedAccount : 'savings'
    };

    window.cxp.mobile.render('root', 'file:///assets', '/portalserver', model, syncPrefs, 'debug', syncPrefs, 'portal-mobile');
```

##API

###cxp.mobile.render(widgetRoot, localContextRoot, remoteContextRoot, portalModel, plugins, logLevel, syncPreferences, portalName)

Render a portal model. This is typically called by a native mobile application that is interacting with a webview

#### Parameters

| Param               | Type   | Description                                       |
|---------------------|--------|---------------------------------------------------|                                                            
| *widgetRoot*        | string | The id of the DOM node to render the portal into  |
| *localContextRoot*  | string | This is typically the base path on the device's filesystem where local static assets (widgets etc) are rendered from. $(contextRoot) placeholders in widget configuration will be replaced with this value |
| *remoteContextRoot* | string | The context root of the remote portal |
| *portalModel*       | Object | A portal model as a Javascript object which is serializable to JSON. This may be a single item or an item tree |
| *plugins*           | Array  | An array of native plugins |
| *logLevel*          | string | The portal client logging level. Possible values are `fatal`. `error`, `warn`, `info`, `debug` and `trace` |
| *syncPreferences*   | Object | A map of synchronized preference names and values |
| *portalName*        | string | The name of the portal being rendered |       

###mobile.render(...)

Android compatible alisa for `cxp.mobile.render(...)`                                                      

###renderWidget(...)

Legacy alias for `cxp.mobile.render(...)`
