/**
 *  ----------------------------------------------------------------
 *  Author : Backbase R&D - Amsterdam - New York
 *  Filename : content.js
 *  Description:
 *
 *  Source code for CXP Universal Content Widget
 *  ----------------------------------------------------------------
 */

b$ = b$ || {};
b$.universal = b$.universal || {};
b$.universal.widgets = b$.universal.widgets || {};
b$.universal.widgets.contentWidget = (function($, bd) {
    'use strict';

    return {
        /**
         * Initialize content widget
         * @param  {Object} oWidget Backbase widget object
         */
        start: function(oWidget) {

            // Extend widget in design mode
            if (be.ice && bd && bd.designMode == 'true') {

                oWidget.iceConfig = be.ice.config;

                var isMasterpage = be.utils.module('top.bd.PageMgmtTree.selectedLink').isMasterPage,
                    isManageable = isMasterpage || (
                        oWidget.model.manageable === 'true' ||
                        oWidget.model.manageable === '' ||
                        oWidget.model.manageable === undefined
                    );

                if (isManageable && be.ice.controller && /(?:ADMIN|CREATOR)/.test(oWidget.model.securityProfile)) {
                    var templateUrl = String(oWidget.getPreference('templateUrl')),
                        enableEditing = function() {

                            // it is possible to swap template for editorial
                            // here is an example for image template
                            if (templateUrl.match(/\/image\.html$/)) {
                                templateUrl = templateUrl.replace(/\/image\.html$/, '/image-editorial.html');
                            }

                            return be.ice.controller.edit(oWidget, templateUrl)
                                .then(function(dom) {
                                    $(oWidget.body).find('.bp-g-include').html(dom);

                                    return dom;
                                });
                        };

                    oWidget.addEventListener('preferenceFormReady', function(event) {
                        var aTemplateVariants = [], selected;
                        var sTemplateVariants = oWidget.getPreference('templateList');
                        var $input = $(event.target.htmlNode).find('input[name="templateUrl"]').hide();

                        if (sTemplateVariants) {
                            var arr = sTemplateVariants.split(',');
                            for (var i = 0; i < arr.length; i += 2) {
                                aTemplateVariants.push({
                                    name: arr[i],
                                    value: arr[i+1]
                                });
                                if (arr[i+1] === $input.val()) {
                                    selected = arr[i];
                                }
                            }

                            var $dd = bc.component.dropdown({
                                target: $input.parent(),
                                label: selected || '-- SELECT VARIANT --',
                                options: aTemplateVariants,
                                selected: $input.val()
                            });

                            $('button[data-action="save"]').on('click', function(){
                                $input.show().val($dd.siblings('select').val());
                            });
                        }
                    });

                    // this widget has property rendering example (template: simple.html)
                    // so we need to refresh widget after property 'title' modified
                    oWidget.model.addEventListener('PrefModified', function(oEvent) {
                        if (oEvent.attrName === 'title') {
                            enableEditing();
                        }
                    }, false, oWidget);

                    return enableEditing();
                }

            } else {
                // Hide broken images on live
                $('img[src=""], img:not([src])', oWidget.body).addClass('be-ice-hide-image');
            }

        }
    };
}(window.jQuery, window.bd));
