'use strict';

var gulp = require('gulp');
var uglify = require('gulp-uglify');
var browserify = require('browserify');
var size = require('gulp-size');
var warnSize = require('gulp-warn-size');
var jshint = require('gulp-jshint');
var source = require('vinyl-source-stream');
var buffer = require('vinyl-buffer');
var del = require('del');
var rename = require('gulp-rename');
var insert = require('gulp-insert');
var util = require('util');
var sourcemaps = require('gulp-sourcemaps');
var gutil = require('gulp-util');
var jasmine = require('gulp-jasmine');

var PACKAGE_MESSAGE =
    '/**\n' +
    ' * %s - Copyright © %s Backbase B.V - All Rights Reserved\n' +
    ' * Version %s\n' +
    ' * %s\n' +
    ' */\n';

gulp.task('clean', function () {
    return del('./dist/**');
});

//these are peer dependencies are redistributed with this package for convenience
var deps = [
    './node_modules/portal5-support/portal5-client-shim.js',
    './node_modules/es6-promise-polyfill-feature/bower_components/es6-promise-polyfill/promise.min.js',
    './src/common/fetch.js',
    './node_modules/intl-feature/bower_components/intl/dist/Intl.complete.js',
    './node_modules/intl-feature/bower_components/intl/dist/Intl.min.js.map'
];

gulp.task('copy-dependencies', ['clean'], function() {
    return gulp.src(deps)
        .pipe(rename(function(path) {
            //the filename used by sdk is wrong. this is a workaround
            if(path.basename === 'portal5-client-shim') {
                path.basename = 'portal-client-shim';
            }
        })).pipe(gulp.dest('./dist'));
});

gulp.task('lint', function () {
    return gulp.src('./src/**/*.js')
        .pipe(jshint())
        .pipe(jshint.reporter('default'))
        .pipe(jshint.reporter('fail'));
});

// unit tests
gulp.task('test', [ 'lint' ],  function () {
    return gulp.src('./spec/**/*-spec.js')
        .pipe(jasmine({
            includeStackTrace: true
        }));
});

//package with browserify
gulp.task('package', ['copy-dependencies'], function() {

    var meta = require('./package.json');
    var year = new Date().getFullYear();
    var description = meta.description;
    var packageMessage = util.format(PACKAGE_MESSAGE, description, year, meta.version, meta.homepage);

    return browserify({
        entries: './src/index.js'
    }).bundle()
        .pipe(source('web-sdk-mobile.js'))
        .pipe(gulp.dest('./dist'))
        .pipe(rename({
            suffix: '.min'
        }))
        .pipe(buffer())
        .pipe(insert.prepend(packageMessage))
        .pipe(sourcemaps.init())
        .pipe(uglify({
            preserveComments: function(node, comment) {
                return comment.value.indexOf('Backbase B.V') > -1;
            }
        }))
        .on('error', gutil.log)
        .pipe(sourcemaps.write('./'))
        .pipe(gulp.dest('./dist'))
        .pipe(size({
            showFiles: true
        }))
        .pipe(warnSize({
            limit: 1000000,
            filter: /\.js$/
        }).on('error', function(err) {
            if(err.plugin === 'gulp-warn-size') {
                process.exit(1);
            }
        }));
});
// remove node_modules folder, it breaks the build
gulp.task('postbuild', [ 'test', 'package' ], function() {
    return del('./node_modules/**');
});


gulp.task('default', [ 'test', 'package' ]);
