var BufferedLogStream = require('../../src/logging/buffered-log-stream');

describe('buffered log stream', function() {
    

    it('can write messages and not exceed the buffer size', function() {

        var BUFFER_SIZE = 10;

        var logStream = new BufferedLogStream(BUFFER_SIZE);
        
        var count = 1;
        while(count <= BUFFER_SIZE) {
            logStream.write({ msg: 'message: ' + count });
            expect(logStream.buffer.length).toBe(count);
            count++;
        }

        while(count <= 20) {
            logStream.write({ msg: 'message: ' + count });
            expect(logStream.buffer.length).toBe(BUFFER_SIZE);
            count++;
        }
    });

    it('can flush the buffer to another log stream', function() {
        
        var myStream1 = {
            out: [],
            write: function(rec) {
                this.out.push(rec);
            }
        };
        var myStream2 = {
            out: [],
            write: function(rec) {
                this.out.push(rec);
            }
        };

        var logStream = new BufferedLogStream(100);
        
        logStream.decorateStream(myStream1);
        logStream.decorateStream(myStream2);

        var count = 1;
        while(count <= 200) {
            logStream.write({ msg: 'message: ' + count });
            count++;
        }
        
        expect(myStream1.out.length).toBe(0);
        expect(myStream2.out.length).toBe(0);
        
        logStream.flush();

        expect(myStream1.out.length).toBe(100);
        expect(myStream2.out.length).toBe(100);
    });

    it('can clear its buffer', function() {
        
        var logStream = new BufferedLogStream(100);
        
        var count = 1;
        while(count <= 200) {
            logStream.write({ msg: 'message: ' + count });
            count++;
        }
        
        expect(logStream.buffer.length).toBe(100);
        logStream.clear();
        expect(logStream.buffer.length).toBe(0);

    });
});

