var rewire = require("rewire");

var ConsolePlainLogStream = rewire('../../src/logging/console-plain-log-stream');


var mockConsole = jasmine.createSpyObj('mockConsole', ['info', 'warn', 'error', 'log'])
ConsolePlainLogStream.__set__({
    console: mockConsole
});

describe('console plain log stream', function() {

    afterEach(function() {
        mockConsole.log.calls.reset();
        mockConsole.info.calls.reset();
        mockConsole.warn.calls.reset();
        mockConsole.error.calls.reset();
    });
    
    it('can write log messages to the console', function () {
        var logStream = new ConsolePlainLogStream();
        
        var rec = {
            name: 'test-logger',
            msg: 'I love mobile'
        };
        
        rec.level = 20;
        logStream.write(rec);
        expect(mockConsole.log).toHaveBeenCalledWith('test-logger: I love mobile');

        rec.level = 30;
        logStream.write(rec);
        expect(mockConsole.info).toHaveBeenCalledWith('test-logger: I love mobile');

        rec.level = 40;
        logStream.write(rec);
        expect(mockConsole.warn).toHaveBeenCalledWith('test-logger: I love mobile');

        rec.level = 50;
        logStream.write(rec);
        expect(mockConsole.error).toHaveBeenCalledWith('test-logger: I love mobile');
    });

    it('can write log messages to the console with the log level and timestamp', function () {
        var logStream = new ConsolePlainLogStream({
            printLevel: true,
            printTimestamp: true
        });

        var rec = {
            name: 'test-logger',
            msg: 'I love mobile',
            time: new Date(2016, 3, 16, 17, 55, 12, 234)
        };

        rec.level = 20;
        rec.levelName = 'LOG';
        logStream.write(rec);
        expect(mockConsole.log.calls.mostRecent().args[0]).toEqual('[17:55:12:0234] LOG: test-logger: I love mobile');

        rec.levelName = 'INFO';
        rec.level = 30;
        logStream.write(rec);
        expect(mockConsole.info.calls.mostRecent().args[0]).toEqual('[17:55:12:0234] INFO: test-logger: I love mobile');

        rec.levelName = 'WARN';
        rec.level = 40;
        logStream.write(rec);
        expect(mockConsole.warn.calls.mostRecent().args[0]).toEqual('[17:55:12:0234] WARN: test-logger: I love mobile');

        rec.levelName = 'FATAL';
        rec.level = 50;
        logStream.write(rec);
        expect(mockConsole.error.calls.mostRecent().args[0]).toEqual('[17:55:12:0234] FATAL: test-logger: I love mobile');
    });
});

