var rewire = require('rewire');
var iosCxpBridge = rewire('../../../src/platforms/ios/ios-bridge');
var jsdom = require('jsdom').jsdom;

var document = jsdom('<html><body</body></html>');
var iframe = document.createElement('iframe');

var mockWindow = {
    cxp: {
        mobile: {}
    }
};
iosCxpBridge.__set__({
    document: document,
    window: mockWindow
});

iosCxpBridge.enable();

describe('communicates to the native side', function() {

    beforeEach(function() {
        spyOn(document, 'createElement').and.returnValue(iframe);
        spyOn(iframe, 'setAttribute');
    });

    afterEach(function() {
        iframe.setAttribute.calls.reset();
    });

    it('notifies that the portal client has loaded', function() {
        expect(mockWindow.cxp.mobile.loaded).toBeDefined();
        mockWindow.cxp.mobile.loaded(123);
        expect(iframe.setAttribute.calls.mostRecent().args[1]).toBe('bb-loaded://?time=123');
    });

    it('instructs the webview to reload', function() {
        expect(mockWindow.cxp.mobile.reload).toBeDefined();
        mockWindow.cxp.mobile.reload();
        expect(iframe.setAttribute.calls.mostRecent().args[1]).toBe('bb-reload://');
    });

    it('instructs the webview to resize to dimensions', function() {
        expect(mockWindow.cxp.mobile.resizeTo).toBeDefined();
        mockWindow.cxp.mobile.resizeTo(100, 150);
        expect(iframe.setAttribute.calls.mostRecent().args[1]).toBe('bb-resize://?w=100&h=150');
    });

    it('instructs the webview to scroll to coords', function() {
        expect(mockWindow.cxp.mobile.scrollTo).toBeDefined();
        mockWindow.cxp.mobile.scrollTo(100, 150);
        expect(iframe.setAttribute.calls.mostRecent().args[1]).toBe('bb-scroll://?x=100&y=150');
    });

    it('publishes an event', function() {
        expect(mockWindow.cxp.mobile.publish).toBeDefined();
        var payload = JSON.stringify({ foo: 'bar'});
        mockWindow.cxp.mobile.publish('foo', payload, 'bar');
        expect(iframe.setAttribute.calls.mostRecent().args[1]).toBe('bb-publish://?event=foo&type=bar&payload=%7B%22foo%22%3A%22bar%22%7D');
    });

    it('subscribes to an event', function() {
        expect(mockWindow.cxp.mobile.subscribe).toBeDefined();
        mockWindow.cxp.mobile.subscribe('foo?');
        expect(iframe.setAttribute.calls.mostRecent().args[1]).toBe('bb-subscribe://?event=foo%3F');
    });

    it('unsubscribes to an event', function() {
        expect(mockWindow.cxp.mobile.unsubscribe).toBeDefined();
        mockWindow.cxp.mobile.unsubscribe('bar?');
        expect(iframe.setAttribute.calls.mostRecent().args[1]).toBe('bb-unsubscribe://?event=bar%3F');
    });
});
