'use strict';

/**
 * Logs records to a buffer until they are flushed to another log stream
 * @param size
 * @constructor
 */
var BufferedLogStream = function(size) {

    this.size = size || 1000;
    this.buffer = [];

    this.decoratedStreams = [];
};

/**
 * Flushing the log will write records to streams added with this method
 * @param stream
 */
BufferedLogStream.prototype.decorateStream = function(stream) {
    this.decoratedStreams.push(stream);
};

/**
 * Write a record to the buffer
 * @param rec
 */
BufferedLogStream.prototype.write = function(rec) {

    if (this.buffer.length >= this.size) {
        this.buffer.shift();
    }

    this.buffer.push(rec);
};

/**
 * Flushes the buffer to a stream
 */
/* jshint ignore:start */
BufferedLogStream.prototype.flush = function() {
    var rec;
    while (rec = this.buffer.shift()) {
        this.decoratedStreams.forEach(function(stream) {
            stream.write(rec);
        });
    }
};
/* jshint ignore:end */

/**
 * Clears the buffer
 */
BufferedLogStream.prototype.clear = function() {
    this.buffer = [];
};

module.exports = BufferedLogStream;