'use strict';

/**
 * Logs messages to the console without any pretty formatting.
 * @param opts
 * @constructor
 */
function ConsolePlainStream(opts) {
    opts = opts || {};
    this.printLevel = !!opts.printLevel;
    this.printTimestamp = !!opts.printTimestamp;
}

/**
 * Write a Bunyan log record
 * @param rec
 */
ConsolePlainStream.prototype.write = function(rec) {

    var loggerName = rec.childName ? rec.name + '/' + rec.childName : rec.name;

    var logMethod;
    if (rec.level < 30) {
        logMethod = 'log';
    } else if (rec.level < 40) {
        logMethod = 'info';
    } else if (rec.level < 50) {
        logMethod = 'warn';
    } else {
        logMethod = 'error';
    }

    function padZeros(number, len) {
        return Array((len + 1) - (number + '').length).join('0') + number;
    }

    function getTimestamp() {
        return '[' +
            padZeros(rec.time.getHours(), 2) + ':' +
            padZeros(rec.time.getMinutes(), 2) + ':' +
            padZeros(rec.time.getSeconds(), 2) + ':' +
            padZeros(rec.time.getMilliseconds(), 4) + ']';
    }
    
    var timestamp = this.printTimestamp ? (getTimestamp() + ' ') : '';
    var level = this.printLevel ? (rec.levelName.toUpperCase() + ': ') : '';
    console[logMethod](timestamp + level + loggerName + ': ' + rec.msg);
};

module.exports = ConsolePlainStream;