'use strict';

var PageRenderer = require('../../render/page-renderer');
var ConsolePlainLogStream = require('../../logging/console-plain-log-stream');

/**
 * Android Page Renderer
 * @constructor
 */
var AndroidPageRenderer = function() {
    PageRenderer.call(this);
};

//
AndroidPageRenderer.prototype = Object.create(PageRenderer.prototype);

AndroidPageRenderer.prototype._createLogStreams = function(logLevel) {

    return [{
        level: logLevel,
        stream: new ConsolePlainLogStream()
    }];
};

AndroidPageRenderer.prototype._renderForPlatform = function (widgetModel, renderer, rootEl) {

    // Page only can be rendered by passing it's children.
    // Page object can't contain features property, thus this check
    // This check is unstable. We should update MBaaS to include a type on items instead
    var model = widgetModel.features ? widgetModel : widgetModel.children;

    renderer.start(model, rootEl).then(function(details) {

        var message = 'Item tree rendered in ' + details.time + 'ms',
            resizeElement = document.getElementsByTagName('html')[0], //the html could contain paddings/margins
            resizeCallback = function() {
                window.cxp.mobile.resize(resizeElement.scrollHeight);
            };
        window.addResizeListener(resizeElement, resizeCallback);
        console.log(message);
        window.cxp.mobile.itemLoaded();
    }).catch(function(err) {
        console.log(err);
    });
};

module.exports = AndroidPageRenderer;
