'use strict';

var window = global.window;  // jshint ignore:line
var document = global.document; // jshint ignore:line

module.exports = {
    enable: function () {

        //force XMLHttpRequests to include a request header
        if(window.XMLHttpRequest) {
            (function(__send) {
                XMLHttpRequest.prototype.send = function(data) {
                    this.setRequestHeader('bb.nsurlrequest.webview', 'true');
                    __send.call(this, data);
                };
            })(XMLHttpRequest.prototype.send);
        }

        function encodeURIExtras(str){
            return encodeURIComponent(str)
                .replace(/[!()*]/g,
                    function(c) { return '%' + c.charCodeAt(0).toString(16);}
                )
                .replace(/[']/g,
                    function(c){ return '\\\''; }
                );
        }

        function addIframe(src) {
            var iframe = document.createElement('IFRAME');
            iframe.setAttribute('src', src);
            document.documentElement.appendChild(iframe);
            iframe.parentNode.removeChild(iframe);
            iframe = null;
        }

        function loaded(time) {
            addIframe('bb-loaded://?time=' + time);
        }

        function reload() {
            addIframe('bb-reload://');
        }

        function resizeTo(width, height) {
            addIframe('bb-resize://?w=' + width + '&h=' + height);
        }

        function scrollTo(x, y) {
            addIframe('bb-scroll://?x=' + x + '&y=' + y);
        }

        function publish(event, payload, eventType) {
            addIframe('bb-publish://?event=' + encodeURIExtras(event) + '&type=' + encodeURIExtras(
                    eventType) + '&payload=' + encodeURIExtras(payload));
        }

        function subscribe(event) {
            addIframe('bb-subscribe://?event=' + encodeURIExtras(event));
        }

        function unsubscribe(event) {
            addIframe('bb-unsubscribe://?event=' + encodeURIExtras(event));
        }

        function executePlugin() {
            if (arguments.length === 0) {
                return;
            }
            var args = [];
            Array.prototype.push.apply(args, arguments);

            var plugin = args.shift();
            var method = args.shift();
            var params = args;
            for (var i = 0; i < params.length; i++) {
                params[i] = encodeURIExtras(params[i]);
            }

            addIframe('bb-plugin://?plugin=' + encodeURIExtras(plugin) + '&method=' +
                encodeURIExtras(method) + '&params=' + params.join('&params='));
        }


        function callPlugin(pluginName, methodName, args, resolve, reject) {
            var _callbackId = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
                var r = Math.random() * 16 | 0, // jshint ignore:line
                    v = c == 'x' ? r : (r & 0x3 | 0x8); // jshint ignore:line
                return v.toString(16);
            });
            var eventName = [pluginName, methodName, _callbackId].join('.');
            var _unsubscribe = function () {
                window.gadgets.pubsub.unsubscribe('plugin.success.' + eventName, _success);
                window.gadgets.pubsub.unsubscribe('plugin.error.' + eventName, _error);
            };
            var _success = function (response) {
                if (!response.keep) {
                    _unsubscribe();
                }
                if (resolve) {
                    resolve(response.data);
                }
            };
            var _error = function (response) {
                if (!response.keep) {
                    _unsubscribe();
                }
                if (reject) {
                    reject(response.data);
                }
            };

            // convert object into JSON.stringify
            for (var i = 0; i < args.length; i++) {
                var arg = args[i];
                if (typeof arg === 'object') {
                    args[i] = JSON.stringify(arg);
                }
            }

            window.gadgets.pubsub.subscribe('plugin.success.' + eventName, _success);
            window.gadgets.pubsub.subscribe('plugin.error.' + eventName, _error);

            executePlugin.apply(window.cxp.mobile, [pluginName, methodName, _callbackId].concat(args));
            return _callbackId;
        }

        window.cxp.mobile = {
            loaded: loaded,
            reload: reload,
            resizeTo: resizeTo,
            scrollTo: scrollTo,
            publish: publish,
            subscribe: subscribe,
            unsubscribe: unsubscribe,
            callPlugin: callPlugin
        };
    }
};
