'use strict';

var PageRenderer = require('../../render/page-renderer');
var ConsolePlainLogStream = require('../../logging/console-plain-log-stream');
var IFrameBridgeLogStream = require('../../logging/iframe-bridge-log-stream');
var BufferedLogStream = require('../../logging/buffered-log-stream');

/**
 * IOS Page Renderer
 * @constructor
 */
var IOSPageRenderer = function() {
    PageRenderer.call(this);
};

//extend PageRenderer
IOSPageRenderer.prototype = Object.create(PageRenderer.prototype);

/**
 * Creates IOS specific log streams
 * @param logLevel
 * @return {*[]}
 * @private
 */
IOSPageRenderer.prototype._createLogStreams = function(logLevel) {

    var consoleStream = new ConsolePlainLogStream({
        printTimestamp: true,
        printLevel: true
    });

    //buffered log stream allows a developer to replay the log by running bufferedLogStream.flush()
    var bufferedLogStream = new BufferedLogStream();
    bufferedLogStream.decorateStream(consoleStream);
    window.bufferedLogStream = bufferedLogStream; //expose globally

    //iframe log stream for logging to native land
    var iframeBridgeLogStream = new IFrameBridgeLogStream();

    //set up log streams
    return [{
        level: logLevel,
        stream: iframeBridgeLogStream
    }, {
        level: logLevel,
        stream: consoleStream
    }, {
        level: logLevel,
        stream: bufferedLogStream
    }];
};

/**
 * Does IOS specific rendering
 * @param widgetModel
 * @param renderer
 * @param rootEl
 * @private
 */
IOSPageRenderer.prototype._renderForPlatform = function(widgetModel, renderer, rootEl) {
    var page = document.getElementsByTagName('html')[0]; //the html could contain paddings/margins
    window.addResizeListener(page, function() {
        window.cxp.mobile.resizeTo(page.scrollWidth, page.scrollHeight);
    });

    // Page only can be rendered by passing it's children.
    // Page object can't contain features property, thus this check
    // This check is unstable. We should update MBaaS to include a type on items instead
    var model = widgetModel.features ? widgetModel : widgetModel.children;

    renderer.start(model, rootEl).then(function(details) {
        window.cxp.mobile.loaded(details.time);
        window.cxp.mobile.resizeTo(page.scrollWidth, page.scrollHeight);
        var message = 'Widget-Engine: Item tree rendered in ' + details.time + 'ms';
        console.log(message);
    }).catch(function(err) {
        console.log(err);
    });
};

module.exports = IOSPageRenderer;
