/* jshint unused: vars */

'use strict';

var window = global.window || {}; // jshint ignore:line

//needed for spec to run
if(!global.window) {
    global.window = window;
}

//core portal
var cxpWebApis = require('cxp-web-apis');

//plugins (these are all functions which should be called to get the plugin instance)
var addFeaturePlugin = require('../plugins/add-feature-plugin');
var exposeChildrenPlugin = require('../plugins/expose-children-plugin');
var mocPlugin = require('../plugins/mobile-optimised-content-plugin');

//retreive csrf tokens using this cookie name
var CSRF_TOKEN_COOKIE_NAME = 'bbCSRF';
//send csrf tokens using this request header name
var CSRF_TOKEN_HEADER_NAME = CSRF_TOKEN_COOKIE_NAME;

/**
 * Renders a page
 * @interface
 * @constructor
 */
var PageRenderer =  function(opts) {
};

/**
 * Starts the rendering process
 * @param rootEl
 * @param model
 * @param pubsub feature implementation
 * @param opts
 */
PageRenderer.prototype.render = function(rootEl, model, pubsub, opts) {

    //options
    opts = opts || {};
    var logLevel = opts.logLevel || 'warn';
    var localContextRoot = opts.localContextRoot || '';
    var remoteContextRoot = opts.remoteContextRoot || '';
    var apiRoot = opts.apiRoot || '';
    var portalName = opts.portalName || 'anonymous-portal';

    //set up config
    var config = cxpWebApis.createConfiguration({
        logStreams: this._createLogStreams(logLevel),
    });
    config.enable('compat');
    config.enable('fullConfigReplacement'); //enables $(contextRoot) replacement for all preferences
    config.set('portalName', portalName);
    config.set('contextRoot', localContextRoot);
    config.set('remoteContextRoot', remoteContextRoot);
    config.set('apiRoot', apiRoot);

    //CSRF
    var csrfTokenCookieValue = null; //TODO: where do we get a CSRF token from?
    if(csrfTokenCookieValue) {
        config.set('csrfToken', {
            name: CSRF_TOKEN_HEADER_NAME,
            value: csrfTokenCookieValue
        });
    }

    //set up renderer
    var renderer = cxpWebApis.getRenderer(config);
    renderer.addPlugin(mocPlugin(config.get('remoteContextRoot')));
    renderer.addPlugin(exposeChildrenPlugin());
    
    //expose CXP as a feature to widgets
    var cxpFeature = {
        name: 'cxp',
        config: config,
        render: renderer,
        model: cxpWebApis.getModel(config)
    };
    renderer.addPlugin(addFeaturePlugin([ cxpFeature.name ]));
    renderer.addFeature(cxpFeature);
    renderer.addFeature(pubsub); //enable the pubsub feature using the native implementation

    this._renderForPlatform(model, renderer, rootEl);
};

/**
 * Should return an array of browser-bunyan log streams.
 * @param opts
 * @private
 */
PageRenderer.prototype._createLogStreams = function(opts) {
    throw new Error('PageRenderer#_createLogStreams must be overridden.');
};

/**
 * Platform specific rendering part
 * @param model
 * @param renderer
 * @param rootEl
 * @private
 */
PageRenderer.prototype._renderForPlatform = function(model, renderer, rootEl) {
    throw new Error('PageRenderer#_renderForPlatform must be overridden.');
};

module.exports = PageRenderer;
